@@ -8,9 +8,9 @@
 ;;;; Date Started:  Apr 2000
 ;;;;
 ;;;;
-;;;; $Id: metaclass.lisp 8079 2003-10-30 04:08:52Z kevin $
+;;;; $Id: metaclass.lisp 11442 2007-01-03 09:49:11Z kevin $
 ;;;;
-;;;; This file is Copyright (c) 2000-2003 by Kevin M. Rosenberg
+;;;; This file is Copyright (c) 2000-2006 by Kevin M. Rosenberg
 ;;;; *************************************************************************
  
 (in-package #:hyperobject)
@@ -25,7 +25,7 @@
     :subobject :hyperlink :hyperlink-parameters :indexed :inverse :unique
     :sql-name :null-allowed :stored :input-filter :unbound-lookup
     :value-constraint :void-text :read-only-groups :hidden-groups :unit
-    :disable-predicate :view-type :list-of-values)
+    :compute-cached-value :disable-predicate :view-type :list-of-values)
   "Slot options that can appear as an initarg")
 
 (defparameter *slot-options-no-initarg*
