@@ -11,9 +11,9 @@
 ;;;; in Text, HTML, and XML formats. This includes hyperlinking
 ;;;; capability and sub-objects.
 ;;;;
-;;;; $Id: mop.lisp 11039 2006-08-18 07:45:31Z kevin $
+;;;; $Id: mop.lisp 11447 2007-01-04 18:12:57Z kevin $
 ;;;;
-;;;; This file is Copyright (c) 2000-2003 by Kevin M. Rosenberg
+;;;; This file is Copyright (c) 2000-2006 by Kevin M. Rosenberg
 ;;;; *************************************************************************
 
 (in-package #:hyperobject)
@@ -51,6 +51,9 @@
    (subobjects :initform nil :accessor subobjects
 	       :documentation
 	       "List of fields that contain a list of subobjects objects.")
+   (compute-cached-values :initform nil :accessor compute-cached-values
+                         :documentation
+                         "List of fields that contain a list of compute-cached-value objects.")
    (hyperlinks :type list :initform nil :accessor hyperlinks
 	       :documentation "List of fields that have hyperlinks")
    (direct-rules :type list :initform nil :initarg :direct-rules
@@ -59,7 +62,7 @@
    (direct-views :type list :initform nil :initarg :direct-views
 		 :accessor direct-views
 		 :documentation "List of views")
-   (class-id :type integer :initform nil
+   (class-id :type integer :initform (+ (* 1000000 (get-universal-time)) (random 1000000))
 	     :accessor class-id
 	     :documentation "Unique ID for the class")
    (default-view :initform nil :initarg :default-view :accessor default-view
@@ -82,11 +85,22 @@
 (defclass subobject ()
   ((name-class :type symbol :initarg :name-class :reader name-class)
    (name-slot :type symbol :initarg :name-slot :reader name-slot)
-   (subobj-class :type symbol :initarg :subobj-class :reader subobj-class)
+   (lazy-class :type symbol :initarg :lazy-class :reader lazy-class)
    (lookup :type (or function symbol) :initarg :lookup :reader lookup)
    (lookup-keys :type list :initarg :lookup-keys :reader lookup-keys))
   (:documentation "subobject information")
-  (:default-initargs :name-class nil :name-slot nil :subobj-class nil
+  (:default-initargs :name-class nil :name-slot nil :lazy-class nil
+		     :lookup nil :lookup-keys nil))
+
+(defclass compute-cached-value ()
+  ((name-class :type symbol :initarg :name-class :reader name-class)
+   (name-slot :type symbol :initarg :name-slot :reader name-slot)
+   (lazy-class :type symbol :initarg :lazy-class
+                              :reader lazy-class)
+   (lookup :type (or function symbol) :initarg :lookup :reader lookup)
+   (lookup-keys :type list :initarg :lookup-keys :reader lookup-keys))
+  (:documentation "subobject information")
+  (:default-initargs :name-class nil :name-slot nil :lazy-class nil
 		     :lookup nil :lookup-keys nil))
 
 
@@ -111,14 +125,100 @@
 (defmethod validate-superclass ((class hyperobject-class) (superclass standard-class))
   t)
 
-(defmethod finalize-inheritance :after ((cl hyperobject-class))
-  ;; Work-around needed for OpenMCL
-  #+ignore
-  (unless (find-class (class-name cl))
-    (setf (find-class (class-name cl)) cl))
 
-  (init-hyperobject-class cl)
-  )
+(declaim (inline delistify))
+(defun delistify (list)
+  "Some MOPs, like openmcl 0.14.2, cons attribute values in a list."
+  (if (listp list)
+      (car list)
+    list))
+
+(defun remove-keyword-arg (arglist akey)
+  (let ((mylist arglist)
+	(newlist ()))
+    (labels ((pop-arg (alist)
+	     (let ((arg (pop alist))
+		   (val (pop alist)))
+	       (unless (equal arg akey)
+		 (setf newlist (append (list arg val) newlist)))
+	       (when alist (pop-arg alist)))))
+      (pop-arg mylist))
+    newlist))
+
+(defun remove-keyword-args (arglist akeys)
+  (let ((mylist arglist)
+	(newlist ()))
+    (labels ((pop-arg (alist)
+	     (let ((arg (pop alist))
+		   (val (pop alist)))
+	       (unless (find arg akeys)
+		 (setf newlist (append (list arg val) newlist)))
+	       (when alist (pop-arg alist)))))
+      (pop-arg mylist))
+    newlist))
+
+
+(defmethod shared-initialize :around ((class hyperobject-class) slot-names
+                                        &rest initargs
+					&key direct-superclasses
+                                        user-name sql-name name description
+					&allow-other-keys)
+  ;(format t "ii ~S ~S ~S ~S ~S~%" initargs base-table direct-superclasses user-name sql-name)
+  (let ((root-class (find-class 'hyperobject nil))
+	(vmc 'hyperobject-class)
+        user-name-plural user-name-str sql-name-str)
+    ;; when does CLSQL pass :qualifier to initialize instance?
+    (setq user-name-str
+          (if user-name
+              (delistify user-name)
+            (and name (format nil "~:(~A~)" name))))
+
+    (setq sql-name-str
+          (if sql-name
+              (delistify sql-name)
+            (and name (lisp-name-to-sql-name name))))
+
+    (if sql-name
+        (delistify sql-name)
+      (and name (lisp-name-to-sql-name name)))
+
+    (setq description (delistify description))
+
+    (setq user-name-plural
+      (if (and (consp user-name) (second user-name))
+          (second user-name)
+        (and user-name-str (format nil "~A~P" user-name-str 2))))
+
+    (flet ((do-call-next-method (direct-superclasses)
+                                (let ((fn-args (list class slot-names :direct-superclasses direct-superclasses))
+                                      (rm-args '(:direct-superclasses)))
+                                  (when user-name-str
+                                    (setq fn-args (nconc fn-args (list :user-name user-name-str)))
+                                    (push :user-name rm-args))
+                                  (when user-name-plural
+                                    (setq fn-args (nconc fn-args (list :user-name-plural user-name-plural)))
+                                    (push :user-name-plural rm-args))
+                                  (when sql-name-str
+                                    (setq fn-args (nconc fn-args (list :sql-name sql-name-str)))
+                                    (push :sql-name rm-args))
+                                  (when description
+                                    (setq fn-args (nconc fn-args (list :description description)))
+                                    (push :description rm-args))
+                                  (setq fn-args (nconc fn-args (remove-keyword-args initargs rm-args)))
+                                  (apply #'call-next-method fn-args))))
+      (if root-class
+          (if (some #'(lambda (super) (typep super vmc))
+                    direct-superclasses)
+              (do-call-next-method direct-superclasses)
+            (do-call-next-method          direct-superclasses #+nil (append (list root-class)
+                                                                            direct-superclasses)))
+	(do-call-next-method direct-superclasses)))))
+
+
+(defmethod finalize-inheritance :after ((cl hyperobject-class))
+  "Initialize a hyperobject class. Calculates all class slots"
+  (finalize-subobjects cl)
+  (finalize-compute-cached cl))
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (when (>= (length (generic-function-lambda-list
@@ -139,16 +239,16 @@
 	      3)
       (pushnew :ho-normal-esdc cl:*features*)))
 
-;; Slot definitions
 (defmethod direct-slot-definition-class ((cl hyperobject-class)
 					 #+ho-normal-dsdc &rest iargs)
+  (declare (ignore iargs))
   (find-class 'hyperobject-dsd))
 
 (defmethod effective-slot-definition-class ((cl hyperobject-class)
 					    #+ho-normal-esdc &rest iargs)
+  (declare (ignore iargs))
   (find-class 'hyperobject-esd))
 
-
 ;;; Slot definitions
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
@@ -233,23 +333,7 @@
     (t
      t)))
 
-(defmethod initialize-instance :around ((obj hyperobject-dsd) &rest initargs)
-  (do* ((saved-initargs initargs)
-        (parsed (list obj))
-        (name (first initargs) (first initargs))
-        (val (second initargs) (second initargs)))
-      ((null initargs)
-       (apply #'call-next-method parsed))
-    (if (eql name :value-type)
-        (progn
-          (setq val (canonicalize-value-type val))
-          (setq parsed (append parsed (list name val)))
-          (setq parsed (append parsed (list :type
-                                            (value-type-to-lisp-type
-                                             (canonicalize-value-type val))))))
-      (setq parsed (append parsed (list name val))))
-    (setq initargs (cddr initargs))))
-  
+
 (defmethod compute-effective-slot-definition :around ((cl hyperobject-class)
 						      #+ho-normal-cesd name
 						      dsds)
@@ -267,7 +351,7 @@
       (setf (esd-sql-length esd) sql-length))
     (setf (esd-user-name esd)
 	  (aif (dsd-user-name dsd)
-	       it
+               it
 	       (string-downcase (symbol-name (slot-definition-name dsd)))))
     (setf (esd-sql-name esd)
 	  (aif (dsd-sql-name dsd)
@@ -278,11 +362,11 @@
 	       it
 	       (lisp-name-to-sql-name (slot-definition-name dsd))))
     (dolist (name '(value-type print-formatter subobject hyperlink
-                    hyperlink-parameters
+                    hyperlink-parameters unbound-lookup
 		    description value-constraint indexed null-allowed
 		    unique short-description void-text read-only-groups
 		    hidden-groups unit disable-predicate view-type
-		    list-of-values stored))
+		    list-of-values compute-cached-value stored))
       (setf (slot-value esd name) (slot-value dsd name)))
     esd))
 
@@ -394,10 +478,11 @@ SQL name"
 ;; The reader is a function and the reader-keys are slot names.  The slot is lazily set to
 ;; the result of applying the function to the slot-values of those slots, and that value
 ;; is also returned.
-(defun ensure-lazy-reader (cl class-name slot-name subobj-class reader
+(defun ensure-lazy-reader (cl class-name slot-name lazy-class reader
 			   &rest reader-keys)
+  (declare (ignore class-name))
   (setf (getf (gethash cl *lazy-readers*) slot-name)
-    (aif subobj-class
+    (aif lazy-class
 	 it
 	 (list* reader (copy-list reader-keys)))))
 
@@ -406,59 +491,41 @@ SQL name"
     nil))
 
 
+(defun store-lazily-computed-objects (cl slot esd-accessor obj-class)
+  (setf (slot-value cl slot)
+        (let ((objs '()))
+          (dolist (slot (class-slots cl))
+            (let-when
+             (def (funcall esd-accessor slot))
+             (let ((obj (make-instance obj-class
+                                       :name-class (class-name cl)
+                                       :name-slot (slot-definition-name slot)
+                                       :lazy-class (when (atom def)
+                                                     def)
+                                       :lookup (when (listp def)
+                                                 (car def))
+                                       :lookup-keys (when (listp def)
+                                                      (cdr def)))))
+               (unless (eq (lookup obj) t)
+                 (apply #'ensure-lazy-reader
+                  cl
+                  (name-class obj) (name-slot obj)
+                  (lazy-class obj)
+                  (lookup obj) (lookup-keys obj))
+                 (push obj objs)))))
+          ;; sbcl/cmu reverse class-slots compared to the defclass form
+          ;; so re-reverse on cmu/sbcl
+          #+(or cmu sbcl) objs
+          #-(or cmu sbcl) (nreverse objs)
+          )))
+
 (defun finalize-subobjects (cl)
-  "Process class subobjects slot"
-  (setf (subobjects cl)
-	(let ((subobjects '()))
-	  (dolist (slot (class-slots cl))
-	    (let-when
-	     (subobj-def (esd-subobject slot))
-	     (let ((subobject
-		    (make-instance 'subobject
-				   :name-class (class-name cl)
-				   :name-slot (slot-definition-name slot)
-				   :subobj-class (when (atom subobj-def)
-						   subobj-def)
-				   :lookup (when (listp subobj-def)
-					     (car subobj-def))
-				   :lookup-keys (when (listp subobj-def)
-						  (cdr subobj-def)))))
-	       (unless (eq (lookup subobject) t)
-		 (apply #'ensure-lazy-reader
-			cl
-			(name-class subobject) (name-slot subobject)
-			(subobj-class subobject)
-			(lookup subobject) (lookup-keys subobject))
-		 (push subobject subobjects)))))
-	  ;; sbcl/cmu reverse class-slots compared to the defclass form
-	  ;; so re-reverse on cmu/sbcl
-	  #+(or cmu sbcl) subobjects
-	  #-(or cmu sbcl) (nreverse subobjects)
-	  )))
+  (store-lazily-computed-objects cl 'subobjects 'esd-subobject 'subobject))
 
-(defun finalize-class-slots (cl)
-  "Make sure all class slots have an expected value"
-  (unless (user-name cl)
-    (setf (user-name cl) (format nil "~:(~A~)" (class-name cl))))
+(defun finalize-compute-cached (cl)
+  (store-lazily-computed-objects cl 'compute-cached-values
+                                 'esd-compute-cached-value 'compute-cached-value))
 
-  (setf (user-name-plural cl)
-	(if (and (consp (user-name cl)) (cadr (user-name cl)))
-	    (cadr (user-name cl))
-	    (format nil "~A~P" (if (consp (user-name cl))
-				   (car (user-name cl))
-				   (user-name cl))
-		    2)))
-
-  (dolist (name '(user-name description version guid sql-name))
-    (awhen (slot-value cl name)
-	   (setf (slot-value cl name)
-	     (if (listp it)
-		 (car it)
-	       it))))
-
-  (unless (sql-name cl)
-    (setf (sql-name cl) (lisp-name-to-sql-name (class-name cl))))
-  )
 
 (defun finalize-documentation (cl)
   "Calculate class documentation slot"
@@ -492,15 +559,15 @@ SQL name"
 
 (defun init-hyperobject-class (cl)
   "Initialize a hyperobject class. Calculates all class slots"
-  (finalize-subobjects cl)
   (finalize-views cl)
   (finalize-hyperlinks cl)
   (finalize-sql cl)
   (finalize-rules cl)
-  (finalize-class-slots cl)
   (finalize-documentation cl))
 
 
+
+
 ;;;; *************************************************************************
 ;;;;  Metaclass Slot Accessors
 ;;;; *************************************************************************
