@@ -7,7 +7,7 @@
 ;;;; Author:   Kevin M. Rosenberg
 ;;;; Created:  Apr 2000
 ;;;;
-;;;; $Id: views.lisp 11085 2006-09-03 02:12:03Z kevin $
+;;;; $Id: views.lisp 11447 2007-01-04 18:12:57Z kevin $
 ;;;;
 ;;;; This file is Copyright (c) 2000-2004 by Kevin M. Rosenberg
 ;;;; *************************************************************************
@@ -65,7 +65,7 @@
    (obj-data-printer :type (or function null) :initform nil
 			:initarg :obj-data-printer
 			:accessor obj-data-printer)
-   (obj-data-print-code :type (or function null) :initform nil
+   (obj-data-print-code :type (or function list null) :initform nil
 		  :initarg :obj-data-print-code
 		  :accessor obj-data-print-code)
    (obj-data-start-printer :type (or function string null) :initform nil
@@ -163,6 +163,7 @@
 				       &rest initargs
 				       &key
 				       &allow-other-keys)
+  (declare (ignore initargs))
   (initialize-view self))
 
 (defun initialize-view (view)
